--------------------------------------------------------------------------------
-- This file is part of the ParaNut project.
-- 
-- Copyright (C) 2022 Christian Meyer <christian.meyer@hs-augsburg.de>
--     Hochschule Augsburg, University of Applied Sciences
--
-- Redistribution and use in source and binary forms, with or without modification,
-- are permitted provided that the following conditions are met:
--
-- 1. Redistributions of source code must retain the above copyright notice, this 
--    list of conditions and the following disclaimer.
--
-- 2. Redistributions in binary form must reproduce the above copyright notice,
--    this list of conditions and the following disclaimer in the documentation and/or
--    other materials provided with the distribution.
--
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
-- ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
-- WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
-- DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
-- ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
-- (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
-- LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
-- ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
-- (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
-- SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
--
-- Description:
--  Translation Lookaside Buffer (TLB) module
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library paranut;
use paranut.memu_lib.all;

entity mtlb_wrapper is
    port (
             clk            : in std_logic;
             reset          : in std_logic;
             -- to MMU...
             flush          : in std_logic;
             -- to PTW
             tlbi           : in tlb_in_type;
             tlbo           : out tlb_out_type
         );
end mtlb_wrapper;

architecture rtl of mtlb_wrapper is

  component MTlb is
    port (
    clk : IN STD_LOGIC;
    reset : IN STD_LOGIC;
    flush : IN STD_LOGIC;
    ptw_req : IN STD_LOGIC;
    ptw_wr : IN STD_LOGIC;
    ptw_va_i : IN STD_LOGIC_VECTOR (19 downto 0);
    ptw_pa_i : IN STD_LOGIC_VECTOR (19 downto 0);
    ptw_superpage_i : IN STD_LOGIC;
    ptw_ac_r_i : IN STD_LOGIC;
    ptw_ac_w_i : IN STD_LOGIC;
    ptw_ac_x_i : IN STD_LOGIC;
    ptw_ac_u_i : IN STD_LOGIC;
    ptw_ac_a_i : IN STD_LOGIC;
    ptw_ac_d_i : IN STD_LOGIC;
    ptw_superpage_o : OUT STD_LOGIC;
    ptw_adr_o : OUT STD_LOGIC_VECTOR (19 downto 0);
    ptw_hit_o : OUT STD_LOGIC;
    ptw_miss_o : OUT STD_LOGIC;
    ptw_ac_r_o : OUT STD_LOGIC;
    ptw_ac_w_o : OUT STD_LOGIC;
    ptw_ac_x_o : OUT STD_LOGIC;
    ptw_ac_u_o : OUT STD_LOGIC;
    ptw_ac_a_o : OUT STD_LOGIC;
    ptw_ac_d_o : OUT STD_LOGIC
    );
  end component;

begin

  HLSMtlb : component MTlb
  port map (
      clk => clk,
      reset => reset,
      -- to MMU
      flush => flush,
      -- to PTW
      ptw_req => tlbi.ptw_req,
      ptw_wr => tlbi.ptw_wr,
      ptw_va_i => tlbi.ptw_va_i,
      ptw_pa_i => tlbi.ptw_pa_i,
      ptw_superpage_i => tlbi.ptw_superpage_i,
      ptw_ac_r_i => tlbi.ptw_ac_r_i,
      ptw_ac_w_i => tlbi.ptw_ac_w_i,
      ptw_ac_x_i => tlbi.ptw_ac_x_i,
      ptw_ac_u_i => tlbi.ptw_ac_u_i,
      ptw_ac_a_i => tlbi.ptw_ac_a_i,
      ptw_ac_d_i => tlbi.ptw_ac_d_i,
      ptw_superpage_o => tlbo.ptw_superpage_o,
      ptw_adr_o => tlbo.ptw_adr_o,
      ptw_hit_o => tlbo.ptw_hit_o,
      ptw_miss_o => tlbo.ptw_miss_o,
      ptw_ac_r_o => tlbo.ptw_ac_r_o,
      ptw_ac_w_o => tlbo.ptw_ac_w_o,
      ptw_ac_x_o => tlbo.ptw_ac_x_o,
      ptw_ac_u_o => tlbo.ptw_ac_u_o,
      ptw_ac_a_o => tlbo.ptw_ac_a_o,
      ptw_ac_d_o => tlbo.ptw_ac_d_o
  );

end rtl;
