--------------------------------------------------------------------------------
-- This file is part of the ParaNut project.
--
-- Copyright (C) 2022-2023 Elias Schuler <elias.schuler@hs-augsburg.de>
--               2022-2023 Lukas Bauer <lukas.bauer@hs-ausgsburg.de>
--     Hochschule Augsburg, University of Applied Sciences
--
-- Redistribution and use in source and binary forms, with or without modification,
-- are permitted provided that the following conditions are met:
--
-- 1. Redistributions of source code must retain the above copyright notice, this
--    list of conditions and the following disclaimer.
--
-- 2. Redistributions in binary form must reproduce the above copyright notice,
--    this list of conditions and the following disclaimer in the documentation and/or
--    other materials provided with the distribution.
--
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
-- ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
-- WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-- DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
-- ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
-- (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
-- LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
-- ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-- (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-- SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
--
-- Description:
--  UART Module
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library paranut;
use paranut.paranut_config.all;
use paranut.memu_lib.all;
use paranut.types.all;
use paranut.uart.all;

entity uart_wrapper is
  port (
    clk    : in std_logic;
    reset  : in std_logic;
    -- to/from Wishbone
    bifwbi : in busif_wishbone_out_type;
    bifwbo : out busif_wishbone_in_type;
    -- External Signals
    baudce_i : IN STD_LOGIC;
    int_o : OUT STD_LOGIC;
    out1_o : OUT STD_LOGIC;
    out2_o : OUT STD_LOGIC;
    rclk_i : IN STD_LOGIC;
    baudoutn_o : OUT STD_LOGIC;
    rtsn_o : OUT STD_LOGIC;
    dtrn_o : OUT STD_LOGIC;
    dcd_i : IN STD_LOGIC;
    rin_i : IN STD_LOGIC;
    sin_i : IN STD_LOGIC;
    sout_o : OUT STD_LOGIC
  );
end uart_wrapper;

architecture rtl of uart_wrapper is
  component Wb8Uart16750 is
  port (
    clk_i : IN STD_LOGIC;
    rst_i : IN STD_LOGIC;
    baudce_i : IN STD_LOGIC;
    wb_cyc_i : IN STD_LOGIC;
    wb_stb_i : IN STD_LOGIC;
    wb_we_i : IN STD_LOGIC;
    wb_adr_i : IN STD_LOGIC_VECTOR (31 downto 0);
    wb_din_i : IN STD_LOGIC_VECTOR (CFG_MEMU_BUSIF_WIDTH-1 downto 0);
    wb_dout_o : OUT STD_LOGIC_VECTOR (CFG_MEMU_BUSIF_WIDTH-1 downto 0);
    wb_ack_o : OUT STD_LOGIC;
    int_o : OUT STD_LOGIC;
    out1_o : OUT STD_LOGIC;
    out2_o : OUT STD_LOGIC;
    rclk_i : IN STD_LOGIC;
    baudoutn_o : OUT STD_LOGIC;
    rtsn_o : OUT STD_LOGIC;
    dtrn_o : OUT STD_LOGIC;
    dcd_i : IN STD_LOGIC;
    rin_i : IN STD_LOGIC;
    sin_i : IN STD_LOGIC;
    sout_o : OUT STD_LOGIC
  );
  end component;


begin

    ICSCUART : component Wb8Uart16750
      port map (
        clk_i => clk,
        rst_i => reset,
        -- WISHBONE input
        wb_stb_i => bifwbi.stb_o,
        wb_cyc_i  => bifwbi.cyc_o,
        wb_we_i => bifwbi.we_o,
        wb_adr_i => bifwbi.adr_o,
        wb_din_i  => bifwbi.dat_o,
        -- WISHBONE output
        wb_ack_o => bifwbo.ack_i,
        wb_dout_o => bifwbo.dat_i,
        -- to all other modules
        baudce_i => baudce_i,
        int_o => int_o,
        out1_o => out1_o,
        out2_o => out2_o,
        rclk_i => rclk_i,
        baudoutn_o => baudoutn_o,
        rtsn_o => rtsn_o,
        dtrn_o => dtrn_o,
        dcd_i => dcd_i,
        rin_i => rin_i,
        sin_i => sin_i,
        sout_o => sout_o
      );


end rtl;
