--------------------------------------------------------------------------------
-- This file is part of the ParaNut project.
--
-- Copyright (C) 2022-2023 Elias Schuler <elias.schuler@hs-augsburg.de>
--               2022-2023 Lukas Bauer <lukas.bauer@hs-ausgsburg.de>
-- 		 Hochschule Augsburg, University of Applied Sciences
--
-- Redistribution and use in source and binary forms, with or without modification,
-- are permitted provided that the following conditions are met:
--
-- 1. Redistributions of source code must retain the above copyright notice, this
--    list of conditions and the following disclaimer.
--
-- 2. Redistributions in binary form must reproduce the above copyright notice,
--    this list of conditions and the following disclaimer in the documentation and/or
--    other materials provided with the distribution.
--
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
-- ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
-- WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-- DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
-- ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
-- (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
-- LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
-- ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-- (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-- SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
--
-- Description:
--  Component and type declarations for the UART Module
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

library paranut;
use paranut.memu_lib.all;
use paranut.paranut_config.all;
use paranut.types.all;

package uart is


	component uart_wrapper is
		port (
		  clk    : in std_logic;
		  reset  : in std_logic;
		  -- to/from Wishbone
		  bifwbi : in busif_wishbone_out_type;
		  bifwbo : out busif_wishbone_in_type;
		  -- External Signals
		  baudce_i : IN STD_LOGIC;
		  int_o : OUT STD_LOGIC;
		  out1_o : OUT STD_LOGIC;
		  out2_o : OUT STD_LOGIC;
		  rclk_i : IN STD_LOGIC;
		  baudoutn_o : OUT STD_LOGIC;
		  rtsn_o : OUT STD_LOGIC;
		  dtrn_o : OUT STD_LOGIC;
		  dcd_i : IN STD_LOGIC;
		  rin_i : IN STD_LOGIC;
		  sin_i : IN STD_LOGIC;
		  sout_o : OUT STD_LOGIC
		);
	  end component;


end package;
